@@ -23,14 +23,14 @@ class Evaluation(Simulation):
 
         self.hhai = honeyhive.HoneyHive(bearer_auth=realign.tracing.honeyhive_key)
         self.eval_name: str = evaluation_name
-        self.dataset_id: str = dataset_id
+        self.hh_dataset_id: str = dataset_id
         self.evaluation_session_ids: List[str] = []
         self.eval_run: Optional[components.CreateRunResponse] = None
 
-        self.dataset = self._load_dataset()
+        self.hh_dataset = self._load_dataset()
         self.query_list = query_list
         self.disable_auto_tracing = True
-        self.runs = len(self.dataset.datapoints) if self.dataset else len(query_list) if query_list else 0
+        self.runs = len(self.hh_dataset.datapoints) if self.hh_dataset else len(query_list) if query_list else 0
 
     def _validate_requirements(self) -> None:
         ''' Sanity check of requirements for HoneyHive evaluations and tracing. '''
@@ -41,23 +41,23 @@ class Evaluation(Simulation):
 
     def _load_dataset(self) -> Optional[Any]:
         ''' Private function to acquire Honeyhive dataset based on dataset_id. '''
-        if not self.dataset_id:
+        if not self.hh_dataset_id:
             return None
         try:
             dataset = self.hhai.datasets.get_datasets(
                 project=realign.tracing.honeyhive_project,
-                dataset_id=self.dataset_id,
+                dataset_id=self.hh_dataset_id,
             )
             if dataset and dataset.object.testcases and len(dataset.object.testcases) > 0:
                 return dataset.object.testcases[0]
         except Exception:
-            raise RuntimeError(f"No dataset found with id - {self.dataset_id} for project - {realign.tracing.honeyhive_project}")
+            raise RuntimeError(f"No dataset found with id - {self.hh_dataset_id} for project - {realign.tracing.honeyhive_project}")
 
     def _get_inputs(self, run_id: int) -> Optional[Dict[str, Any]]:
         ''' Private function to process and iterate over HoneyHive datapoints from Honeyhive dataset '''
-        if self.dataset and self.dataset.datapoints and len(self.dataset.datapoints) > 0 :
+        if self.hh_dataset and self.hh_dataset.datapoints and len(self.hh_dataset.datapoints) > 0 :
             try:
-                datapoint_id = self.dataset.datapoints[run_id]
+                datapoint_id = self.hh_dataset.datapoints[run_id]
                 datapoint_response = self.hhai.datapoints.get_datapoint(id=datapoint_id)
                 return datapoint_response.object.datapoint[0].inputs
             except Exception as e:
@@ -89,9 +89,9 @@ class Evaluation(Simulation):
                 "run_id": self.eval_run.run_id,
                 "inputs": inputs
             }
-            if self.dataset:
-                tracing_metadata["datapoint_id"] = self.dataset.datapoints[run_id]
-                tracing_metadata["dataset_id"] = self.dataset_id
+            if self.hh_dataset:
+                tracing_metadata["datapoint_id"] = self.hh_dataset.datapoints[run_id]
+                tracing_metadata["dataset_id"] = self.hh_dataset_id
             if evaluation_output:
                 tracing_metadata["outputs"] = evaluation_output
 
@@ -118,7 +118,7 @@ class Evaluation(Simulation):
         eval_run = self.hhai.runs.create_run(request=components.CreateRunRequest(
             project=realign.tracing.honeyhive_project,
             name=self.eval_name,
-            dataset_id=self.dataset_id,
+            dataset_id=self.hh_dataset_id,
             event_ids=[],
         ))
         self.eval_run = eval_run.create_run_response
